@@ -16,9 +16,9 @@ use Locale::Maketext::Simple    Style => 'gettext';
 
 ### solaris has silly /bin/tar output ###
 use constant ON_SOLARIS     => $^O eq 'solaris' ? 1 : 0;
-use constant ON_NETBSD      => $^O eq 'netbsd' ? 1 : 0;
-use constant ON_OPENBSD     => $^O eq 'openbsd' ? 1 : 0;
-use constant ON_FREEBSD     => $^O =~ m!^(free|midnight)bsd$! ? 1 : 0;
+use constant ON_NETBSD      => $^O =~ m!^(netbsd|minix)$! ? 1 : 0;
+use constant ON_OPENBSD     => $^O =~ m!^(openbsd|bitrig)$! ? 1 : 0;
+use constant ON_FREEBSD     => $^O =~ m!^(free|midnight|dragonfly)(bsd)?$! ? 1 : 0;
 use constant ON_LINUX       => $^O eq 'linux' ? 1 : 0;
 use constant FILE_EXISTS    => sub { -e $_[0] ? 1 : 0 };
 
@@ -48,18 +48,19 @@ use vars qw[$VERSION $PREFER_BIN $PROGRAMS $WARN $DEBUG
             $_ALLOW_BIN $_ALLOW_PURE_PERL $_ALLOW_TAR_ITER
          ];
 
-$VERSION            = '0.66';
+$VERSION            = '0.84';
 $PREFER_BIN         = 0;
 $WARN               = 1;
 $DEBUG              = 0;
 $_ALLOW_PURE_PERL   = 1;    # allow pure perl extractors
 $_ALLOW_BIN         = 1;    # allow binary extractors
-$_ALLOW_TAR_ITER        = 1;    # try to use Archive::Tar->iter if available
+$_ALLOW_TAR_ITER    = 1;    # try to use Archive::Tar->iter if available
 
 # same as all constants
 my @Types           = ( TGZ, TAR, GZ, ZIP, BZ2, TBZ, Z, LZMA, XZ, TXZ );
 
 local $Params::Check::VERBOSE = $Params::Check::VERBOSE = 1;
+local $Module::Load::Conditional::FORCE_SAFE_INC = 1;
 
 =pod
 
@@ -130,19 +131,21 @@ See the C<HOW IT WORKS> section further down for details.
 ### see what /bin/programs are available ###
 $PROGRAMS = {};
 CMD: for my $pgm (qw[tar unzip gzip bunzip2 uncompress unlzma unxz]) {
-    if ( $pgm eq 'unzip' and ( ON_NETBSD or ON_FREEBSD ) ) {
-      local $IPC::Cmd::INSTANCES = 1;
-      ($PROGRAMS->{$pgm}) = grep { ON_NETBSD ? m!/usr/pkg/! : m!/usr/local! } can_run($pgm);
+    if ( $pgm eq 'unzip' and ON_FREEBSD and my $unzip = can_run('info-unzip') ) {
+      $PROGRAMS->{$pgm} = $unzip;
       next CMD;
     }
-    if ( $pgm eq 'unzip' and ON_LINUX ) {
-      # Check if 'unzip' is busybox masquerading
+    if ( $pgm eq 'unzip' and ( ON_FREEBSD || ON_LINUX ) ) {
       local $IPC::Cmd::INSTANCES = 1;
-      my $opt = ON_VMS ? '"-Z"' : '-Z';
-      ($PROGRAMS->{$pgm}) = grep { scalar run(command=> [ $_, $opt, '-1' ]) } can_run($pgm);
+      ($PROGRAMS->{$pgm}) = grep { _is_infozip_esque($_) } can_run($pgm);
       next CMD;
     }
-    if ( $pgm eq 'tar' and ON_OPENBSD ) {
+    if ( $pgm eq 'unzip' and ON_NETBSD ) {
+      local $IPC::Cmd::INSTANCES = 1;
+      ($PROGRAMS->{$pgm}) = grep { m!/usr/pkg/! } can_run($pgm);
+      next CMD;
+    }
+    if ( $pgm eq 'tar' and ( ON_OPENBSD || ON_SOLARIS || ON_NETBSD ) ) {
       # try gtar first
       next CMD if $PROGRAMS->{$pgm} = can_run('gtar');
     }
@@ -647,7 +650,7 @@ sub have_old_bunzip2 {
 ### as a remote shell, and the extract fails.
 {   my @ExtraTarFlags;
     if( ON_WIN32 and my $cmd = __PACKAGE__->bin_tar ) {
-
+        $cmd = $1 if $cmd =~ m{^(.+)}s; # Tainted perl #
         ### if this is gnu tar we are running, we need to use --force-local
         push @ExtraTarFlags, '--force-local' if `$cmd --version` =~ /gnu tar/i;
     }
@@ -691,7 +694,7 @@ sub have_old_bunzip2 {
         ### depending whether you run with -x or -t
         ### compensate for this insanity by running -t first, then -x
         {    my $cmd =
-                $self->is_tgz ? [$self->bin_gzip, '-cdf', $archive, '|',
+                $self->is_tgz ? [$self->bin_gzip, '-c', '-d', '-f', $archive, '|',
                                  $self->bin_tar, '-tf', '-'] :
                 $self->is_tbz ? [$self->bin_bunzip2, '-cd', $archive, '|',
                                  $self->bin_tar, '-tf', '-'] :
@@ -746,7 +749,7 @@ sub have_old_bunzip2 {
 
         ### now actually extract it ###
         {   my $cmd =
-                $self->is_tgz ? [$self->bin_gzip, '-cdf', $archive, '|',
+                $self->is_tgz ? [$self->bin_gzip, '-c', '-d', '-f', $archive, '|',
                                  $self->bin_tar, '-xf', '-'] :
                 $self->is_tbz ? [$self->bin_bunzip2, '-cd', $archive, '|',
                                  $self->bin_tar, '-xf', '-'] :
@@ -812,8 +815,11 @@ sub _untar_at {
     ### if A::T's version is 0.99 or higher
     if( $self->is_tgz ) {
         my $use_list = { 'Compress::Zlib' => '0.0' };
+        {
+           local $@;
            $use_list->{ 'IO::Zlib' } = '0.0'
-                if $Archive::Tar::VERSION >= '0.99';
+                if eval { Archive::Tar->VERSION('0.99'); 1 };
+        }
 
         unless( can_load( modules => $use_list ) ) {
             my $which = join '/', sort keys %$use_list;
@@ -964,7 +970,7 @@ sub _gunzip_bin {
         return $self->_error(loc("Could not open '%1' for writing: %2",
                             $self->_gunzip_to, $! ));
 
-    my $cmd = [ $self->bin_gzip, '-cdf', $self->archive ];
+    my $cmd = [ $self->bin_gzip, '-c', '-d', '-f', $self->archive ];
 
     my $buffer;
     unless( scalar run( command => $cmd,
@@ -1107,8 +1113,8 @@ sub _unzip_bin {
             ### Annoyingly, pesky MSWin32 can either have 'native' tools
             ### which have \r\n line endings or Cygwin-based tools which
             ### have \n line endings. Jan Dubois suggested using this fix
-            local $/ = ON_WIN32 ? qr/\r?\n/ : "\n";
-            $self->files( [split $/, $buffer] );
+            my $split = ON_WIN32 ? qr/\r?\n/ : "\n";
+            $self->files( [split $split, $buffer] );
         }
     }
 
@@ -1360,7 +1366,7 @@ sub _unxz_bin {
         return $self->_error(loc("Could not open '%1' for writing: %2",
                             $self->_gunzip_to, $! ));
 
-    my $cmd = [ $self->bin_unxz, '-cdf', $self->archive ];
+    my $cmd = [ $self->bin_unxz, '-c', '-d', '-f', $self->archive ];
 
     my $buffer;
     unless( scalar run( command => $cmd,
@@ -1499,6 +1505,44 @@ sub _unlzma_cz {
     return 1;
 }
 
+#####################################
+#
+# unzip heuristics for FreeBSD-alikes
+#
+#####################################
+
+sub _is_infozip_esque {
+  my $unzip = shift;
+
+  my @strings;
+  my $buf = '';
+
+  {
+    open my $file, '<', $unzip or die "$!\n";
+    binmode $file;
+    local $/ = \1;
+    local $_;
+    while(<$file>) {
+      if ( m![[:print:]]! ) {
+        $buf .= $_;
+        next;
+      }
+      if ( $buf and m![^[:print:]]! ) {
+        push @strings, $buf if length $buf >= 4;
+        $buf = '';
+        next;
+      }
+    }
+  }
+  push @strings, $buf if $buf;
+  foreach my $part ( @strings ) {
+    if ( $part =~ m!ZIPINFO! or $part =~ m!usage:.+?Z1! ) {
+      return $unzip;
+    }
+  }
+  return;
+}
+
 #################################
 #
 # Error code
